"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const CommandUtil_1 = require("./CommandUtil");
const Logger_1 = require("../Logger");
const ProjectType_1 = require("../microclimate/project/ProjectType");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
function containerShellCmd(project) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.Log.d("containerBashCmd invoked");
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject(...ProjectState_1.ProjectState.getEnabledStates());
            if (selected == null) {
                // user cancelled
                Logger_1.Log.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        if (project.containerID == null || project.containerID === "") {
            vscode.window.showWarningMessage(translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "noContainerForShell", { projectName: project.name }));
            return;
        }
        // annoyingly, only python project containers seem to not have bash installed.
        // TODO We could check what's installed by doing docker exec through child_process, if that's worth the trouble.
        const toExec = project.type.type === ProjectType_1.ProjectType.Types.PYTHON ? "sh" : "bash"; // non-nls
        // const env = convertNodeEnvToTerminalEnv(process.env);
        const options = {
            name: `${toExec} - ${project.name}`,
        };
        const term = vscode.window.createTerminal(options);
        term.sendText(`docker exec -it ${project.containerID} /usr/bin/env ${toExec}`); // non-nls
        term.show();
    });
}
exports.default = containerShellCmd;
/*
async function getExistingTerminals(name: string): Promise<vscode.Terminal[] | undefined> {
    //const matchingTerms: vscode.Terminal[] = vscode.window.terminals.filter( (term) => term.name === name);
    return vscode.window.terminals.filter( (term) => term.name === name);
}*/
/*
// The format required for environment variables to be passed a vscode terminal
interface TerminalEnv {
    [key: string]: string | null;
}*/
/**
 * Convert a NodeJS.ProcessEnv to the slightly different format VSCode requires -
 * This actually only consists of replacing 'undefined' values with 'null'.
 */
/*
function convertNodeEnvToTerminalEnv(nodeEnv: NodeJS.ProcessEnv): TerminalEnv {
    // Create an empty object, then loop over the key/values of the NodeEnv.
    // If the value is not undefined, set the new k/v into the new object.
    // if it is undefined, set key=null into the new object. Then return that reconstructed object.

    return Object.keys(nodeEnv).reduce( (result: TerminalEnv, key): {} => {
        let value: string | null = nodeEnv[key] || null;
        if (value === undefined) {
            // Replace 'undefined' with 'null' because that is what TerminalOptions requires
            value = null;
        }
        result[key] = value;
        return result;
    }, {});
}*/
//# sourceMappingURL=ContainerShellCmd.js.map