"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
const Logger_1 = require("../Logger");
const STRING_NS = StringNamespaces_1.default.LOGS;
function hideLogsCmd() {
    return __awaiter(this, void 0, void 0, function* () {
        const quickPickItems = ConnectionManager_1.default.instance.connections.reduce((logs, connection) => {
            return logs.concat(connection.logManager.getAllOpenLogs());
        }, []);
        if (quickPickItems.length === 0) {
            vscode.window.showInformationMessage(translator_1.default.t(STRING_NS, "noLogsToHide"));
            return;
        }
        const options = {
            canPickMany: true
        };
        // https://github.com/Microsoft/vscode/issues/64014
        // const selection: vscode.QuickPickItem[] | undefined = await vscode.window.showQuickPick(quickPickItems, options);
        // The return type is an array of MCLogs, but we have to mark it as 'any' due to issue linked above
        const selection = yield vscode.window.showQuickPick(quickPickItems, options);
        if (selection != null) {
            // Log.d("selection", selection);
            const logsToHide = selection;
            Logger_1.default.d(`Hiding ${logsToHide.length} logs`);
            logsToHide.forEach((log) => {
                log.destroy();
            });
            // vscode.window.showInformationMessage(Translator.t(STRING_NS, "hidNLogs", { count: logsToHide.length }));
        }
    });
}
exports.default = hideLogsCmd;
//# sourceMappingURL=HideLogsCmd.js.map