"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Project_1 = require("../microclimate/project/Project");
const Connection_1 = require("../microclimate/connection/Connection");
const CommandUtil_1 = require("./CommandUtil");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const STRING_NS = StringNamespaces_1.default.CMD_OPEN_IN_BROWSER;
function openInBrowserCmd(resource) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.Log.d("OpenInBrowserCmd invoked");
        if (resource == null) {
            const selected = yield CommandUtil_1.promptForResource(true, ProjectState_1.ProjectState.AppStates.STARTED);
            if (selected == null) {
                Logger_1.Log.d("User cancelled prompt for resource");
                // user cancelled
                return;
            }
            resource = selected;
        }
        let uriToOpen;
        // This will open the project or Microclimate in the external web browser.
        if (resource instanceof Project_1.default) {
            const project = resource;
            if (!project.state.isStarted) {
                vscode.window.showWarningMessage(translator_1.default.t(STRING_NS, "canOnlyOpenStartedProjects"));
                return;
            }
            else if (project.appBaseUrl == null) {
                Logger_1.Log.e("Project is started but has no appBaseUrl: " + project.name);
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "failedDetermineAppUrl", { projectName: project.name }));
                return;
            }
            uriToOpen = project.appBaseUrl;
        }
        else if (resource instanceof Connection_1.default) {
            const conn = resource;
            if (!conn.isConnected) {
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "cantOpenDisconnected"));
                return;
            }
            uriToOpen = conn.mcUri;
        }
        else {
            // should never happen
            Logger_1.Log.e(`Don't know how to open object of type ${typeof (resource)} in browser`);
            return;
        }
        Logger_1.Log.i("Open in browser: " + uriToOpen);
        // vscode.window.showInformationMessage("Opening " + uriToOpen);
        vscode.commands.executeCommand(Commands_1.default.VSC_OPEN, uriToOpen);
    });
}
exports.default = openInBrowserCmd;
//# sourceMappingURL=OpenInBrowserCmd.js.map