"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Logger_1 = require("../Logger");
const CommandUtil_1 = require("./CommandUtil");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const STRING_NS = StringNamespaces_1.default.CMD_OPEN_LOG;
/**
 *
 * @param isAppLog - Indicates whether the user wants to open the App log or Build log.
 */
function openLogCmd(project, isAppLog) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("OpenLogCmd invoked");
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject(...ProjectState_1.default.getEnabledStates());
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        let log;
        if (isAppLog) {
            if (!project.state.isEnabled) {
                // If we were to create an app log for a disabled project,
                // it would just say "waiting for Microclimate to send logs" until the app starts.
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "noLogsForDisabled"));
                return;
            }
            else if (!project.state.isStarted) {
                vscode.window.showWarningMessage(translator_1.default.t(STRING_NS, "projectIsNotStarted", { projectName: project.name, projectState: project.state.appState }));
            }
            log = project.connection.logManager.getOrCreateAppLog(project.id, project.name);
        }
        else {
            if (!project.type.providesBuildLog) {
                vscode.window.showErrorMessage(translator_1.default.t(STRING_NS, "noBuildLogsForType", { projectType: project.type.type }));
                return;
            }
            log = project.connection.logManager.getOrCreateBuildLog(project.id, project.name);
        }
        log.showOutputChannel();
    });
}
exports.default = openLogCmd;
//# sourceMappingURL=OpenLogCmd.js.map