"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const Project_1 = require("../microclimate/project/Project");
const Connection_1 = require("../microclimate/connection/Connection");
const CommandUtil_1 = require("./CommandUtil");
const Logger_1 = require("../Logger");
const Commands_1 = require("../constants/Commands");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
function openWorkspaceFolderCmd(resource) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d(`Go to folder command invoked on ${resource}`);
        if (resource == null) {
            const selected = yield CommandUtil_1.promptForResource(false);
            if (selected == null) {
                Logger_1.default.d("User cancelled prompt for resource");
                // user cancelled
                return;
            }
            resource = selected;
        }
        let uri;
        if (resource instanceof Project_1.default) {
            uri = resource.localPath;
        }
        else if (resource instanceof Connection_1.default) {
            uri = resource.workspacePath;
        }
        else {
            // Should never happen
            Logger_1.default.e(`Could not get resource URI from object of type ${typeof (resource)}:`, resource);
            return;
        }
        Logger_1.default.i("Going to folder " + uri);
        const currentFolders = vscode.workspace.workspaceFolders;
        // currentFolders[0] is the current workspace root.
        if (currentFolders != null && currentFolders[0] != null && currentFolders[0].uri.fsPath === uri.fsPath) {
            Logger_1.default.i("Selected folder is already workspace root, nothing to do");
            vscode.window.showWarningMessage(translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "alreadyInSelectedFolder"));
        }
        else {
            // To change 'in new window' behaviour, use "window.openFoldersInNewWindow": "default",
            Logger_1.default.i(`Opening folder ${uri}`);
            vscode.commands.executeCommand(Commands_1.default.VSC_OPEN_FOLDER, uri);
        }
    });
}
exports.default = openWorkspaceFolderCmd;
//# sourceMappingURL=OpenWorkspaceFolderCmd.js.map