"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const CommandUtil_1 = require("./CommandUtil");
const Logger_1 = require("../Logger");
const Requester_1 = require("../microclimate/project/Requester");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const ProjectState_1 = require("../microclimate/project/ProjectState");
function requestBuildCmd(project) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("RequestBuildCmd invoked");
        if (project == null) {
            const selected = yield CommandUtil_1.promptForProject(...ProjectState_1.default.getEnabledStates());
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        if (project.state.isBuilding) {
            vscode.window.showWarningMessage(translator_1.default.t(StringNamespaces_1.default.CMD_MISC, "projectAlreadyBuilding", { projectName: project.name }));
            return;
        }
        /*
        if (project.autoBuildEnabled) {
            vscode.window.showWarningMessage(Translator.t(StringNamespaces.CMD_MISC, "explicitBuildNotNecessary", { projectName: project.name }));
            // still do the build, though.
        }*/
        Logger_1.default.i(`Request build for project ${project.name}`);
        Requester_1.default.requestBuild(project);
    });
}
exports.default = requestBuildCmd;
//# sourceMappingURL=RequestBuildCmd.js.map