"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const CommandUtil_1 = require("./CommandUtil");
const ProjectState_1 = require("../microclimate/project/ProjectState");
const Logger_1 = require("../Logger");
const Requester_1 = require("../microclimate/project/Requester");
/**
 * @param enable - Whether the given project is to be enabled, or disabled.
 * IE if enable=true, the command will only succeed if the project is disabled when the command is invoked.
 */
function toggleEnablementCmd(project, enable) {
    return __awaiter(this, void 0, void 0, function* () {
        Logger_1.default.d("ToggleEnablementCmd invoked");
        if (project == null) {
            let acceptStates;
            if (enable) {
                acceptStates = [ProjectState_1.default.AppStates.DISABLED];
            }
            else {
                acceptStates = ProjectState_1.default.getEnabledStates();
            }
            const selected = yield CommandUtil_1.promptForProject(...acceptStates);
            if (selected == null) {
                // user cancelled
                Logger_1.default.d("User cancelled project prompt");
                return;
            }
            project = selected;
        }
        Logger_1.default.i(`Toggle enablement for project ${project.name}`);
        return Requester_1.default.requestToggleEnablement(project);
    });
}
exports.default = toggleEnablementCmd;
//# sourceMappingURL=ToggleEnablementCmd.js.map