"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
// non-nls-file
/**
 * Class to hold constants that are Portal API endpoint paths
 */
class Endpoints {
    static getEndpoint(connection, endpoint) {
        return connection.mcUri.toString().concat(endpoint);
    }
    static getProjectEndpoint(connection, projectID, endpoint) {
        return connection.mcUri.toString().concat(`${Endpoints.PROJECTS}/${projectID}/${endpoint}`);
    }
    static getAppMonitorUrl(connection, projectID) {
        return connection.mcUri.with({ query: `project=${projectID}&view=monitor` });
    }
    static getCreateOrImportUrl(connection, create) {
        const query = create ? this.QUERY_NEW_PROJECT : this.QUERY_IMPORT_PROJECT;
        return connection.mcUri.with({ query });
    }
    static getEnablementAction(enable) {
        return `${enable ? "open" : "close"}`;
    }
}
// "regular" endpoints, eg "localhost:9090/api/v1/environment"
Endpoints.ENVIRONMENT = "api/v1/environment";
Endpoints.PROJECTS = "api/v1/projects";
// Deprecated
Endpoints.VALIDATE_OLD = "api/v1/validate";
// Deprecated
Endpoints.GENERATE_OLD = "api/v1/validate/generate";
// Project endpoints, eg "localhost:9090/api/v1/project/81eba580-0aea-11e9-b530-67b2995d0cd9/restart"
Endpoints.RESTART_ACTION = "restart";
Endpoints.BUILD_ACTION = "build";
Endpoints.BUILD_LOG = "build-log";
Endpoints.VALIDATE = "validate";
Endpoints.GENERATE = "validate/generate";
Endpoints.QUERY_NEW_PROJECT = "new-project=true";
Endpoints.QUERY_IMPORT_PROJECT = "import-project=true";
exports.default = Endpoints;
//# sourceMappingURL=Endpoints.js.map