"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const ProjectType_1 = require("../microclimate/project/ProjectType");
const Logger_1 = require("../Logger");
// non-nls-file
// from https://github.ibm.com/dev-ex/microclimate/blob/master/docker/file-watcher/server/src/projects/constants.ts
var StartModes;
(function (StartModes) {
    let Modes;
    (function (Modes) {
        Modes["RUN"] = "run";
        Modes["DEBUG"] = "debug";
        Modes["DEBUG_NO_INIT"] = "debugNoInit";
    })(Modes = StartModes.Modes || (StartModes.Modes = {}));
    function getUserFriendlyStartMode(startMode) {
        switch (startMode) {
            case Modes.RUN:
                return "run";
            case Modes.DEBUG:
            // For now, debug vs debugNoInit is not exposed to the user. So in both cases it's just "Debug" to them.
            // return "debug (with initial break)";
            case Modes.DEBUG_NO_INIT:
                return "debug";
            default:
                Logger_1.default.e(`Unknown start mode "${startMode}"!`);
                return "unknown";
        }
    }
    StartModes.getUserFriendlyStartMode = getUserFriendlyStartMode;
    function allStartModes() {
        return [
            Modes.RUN,
            Modes.DEBUG,
            Modes.DEBUG_NO_INIT
        ];
    }
    StartModes.allStartModes = allStartModes;
    function isDebugMode(startMode) {
        return startMode === Modes.DEBUG.toString() || startMode === Modes.DEBUG_NO_INIT.toString();
    }
    StartModes.isDebugMode = isDebugMode;
    function getDefaultStartMode(debug, projectType) {
        if (!debug) {
            return Modes.RUN;
        }
        if (projectType === ProjectType_1.default.Types.MICROPROFILE) {
            return Modes.DEBUG;
        }
        return Modes.DEBUG_NO_INIT;
    }
    StartModes.getDefaultStartMode = getDefaultStartMode;
})(StartModes || (StartModes = {}));
exports.default = StartModes;
//# sourceMappingURL=StartModes.js.map