"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const i18next = require("i18next");
const i18nextBackend = require("i18next-node-fs-backend");
const path = require("path");
const Logger_1 = require("../../Logger");
class Translator {
    /**
     * Use i18next to translate the given string as "$namespace.key".
     * Must explicitly call init before calling this.
     */
    static t(namespace, key, options) {
        const hasNamespace = namespace != null && namespace.length > 0;
        const fullKey = hasNamespace ? `${namespace}.${key}` : key;
        if (this._t == null) {
            // init has not been called, or there was an init error
            Logger_1.default.e(`i18next was not initialized, returning key "${fullKey}"`);
            return fullKey;
        }
        // _t returns 'any', but seems to always be string. Handle that here so that caller can assume it's a string.
        const tResult = this._t(fullKey, options);
        if (typeof tResult === typeof "") {
            if (tResult === fullKey) {
                Logger_1.default.e(`Did not find string with key: ${fullKey}`);
            }
            return tResult;
        }
        else {
            // Don't think this will ever happen
            Logger_1.default.e(`Unexpected result from translation function, type is ${typeof tResult}, result is:`, tResult);
            return fullKey;
        }
    }
    /**
     * Must call this before calling t
     */
    static init() {
        return new Promise((resolve, reject) => {
            i18next
                .use(i18nextBackend) // required so we can load strings from filesystem
                .init({
                // only support english
                lng: "en",
                fallbackLng: "en",
                // debug: true,
                saveMissing: true,
                backend: {
                    loadPath: path.join(__dirname, "strings-{{lng}}.json")
                }
            }, (err, translationFn) => {
                if (err != null) {
                    return reject(err);
                }
                else {
                    Logger_1.default.i("i18next initialized");
                    this._t = translationFn;
                    return resolve(translationFn);
                }
            });
        });
    }
}
exports.default = Translator;
//# sourceMappingURL=translator.js.map