/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const CommandUtil_1 = require("./command/CommandUtil");
const InitViews_1 = require("./view/InitViews");
const Logger_1 = require("./Logger");
const translator_1 = require("./constants/strings/translator");
const StringNamespaces_1 = require("./constants/strings/StringNamespaces");
// this method is called when your extension is activated
// your extension is activated the very first time the command is executed
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        process.on("unhandledRejection", (err) => Logger_1.default.e("Unhandled promise rejection:", err));
        // Initialize our globals
        global.__extRoot = context.extensionPath;
        // Declared as 'any' type, but will always be assigned globalState which is a vscode.Memento
        global.extGlobalState = context.globalState;
        Logger_1.default.setLogFilePath(context);
        Logger_1.default.i("Finished activating logger");
        try {
            yield translator_1.default.init();
        }
        catch (err) {
            // This string can't be translated for obvious reasons :)
            const errmsg = "Error initializing i18next - placeholder strings will be used! " + (err.message || err); // non-nls
            Logger_1.default.e(errmsg, err);
            vscode.window.showErrorMessage(errmsg);
        }
        const msg = translator_1.default.t(StringNamespaces_1.default.DEFAULT, "activeMsg");
        // Make sure i18next loaded the strings properly here.
        Logger_1.default.i("activeMsg:", msg);
        // vscode.window.showInformationMessage(msg);
        ignoreMCFiles();
        const subscriptions = [
            ...InitViews_1.default(),
            ...CommandUtil_1.createCommands(),
        ];
        subscriptions.forEach((e) => {
            // Logger.log("Adding subscription " + util.inspect(e));
            context.subscriptions.push(e);
        });
    });
}
exports.activate = activate;
// this method is called when your extension is deactivated
function deactivate() {
    // nothing here
}
exports.deactivate = deactivate;
const excludeSection = "exclude"; // non-nls
const prePattern = "**/"; // non-nls
// files or directories, doesn't matter, trailing / not required.
const filesToIgnore = [
    ".Trash-0",
    ".config",
    ".extensions",
    ".idc",
    ".license-accept",
    ".logs",
    ".nyc_output",
    ".projects" // non-nls
];
/**
 * Add to the user's `files.exclude` setting to exclude a bunch of files
 * in the microclimate-workspace that the user probably doesn't want to see.
 */
function ignoreMCFiles() {
    return __awaiter(this, void 0, void 0, function* () {
        if (!inMCWorkspace()) {
            Logger_1.default.d("Not ignoring Microclimate files, not in a microclimate-workspace");
            return;
        }
        Logger_1.default.d("Ignoring Microclimate files");
        const filesConfig = vscode.workspace.getConfiguration("files", null); // non-nls
        const existing = filesConfig.get(excludeSection) || {};
        filesToIgnore.forEach((toIgnore) => {
            const newIgnore = prePattern + toIgnore;
            // If the user already set it to false, don't undo that!
            if (existing[newIgnore] == null) {
                existing[newIgnore] = true;
            }
        });
        filesConfig.update(excludeSection, existing, vscode.ConfigurationTarget.Workspace);
    });
}
function inMCWorkspace() {
    const wsFolders = vscode.workspace.workspaceFolders;
    if (wsFolders != null) {
        return wsFolders.some((folder) => folder.uri.fsPath.endsWith("microclimate-workspace")); // non-nls
    }
    else {
        return false;
    }
}
//# sourceMappingURL=extension.js.map