"use strict";
/*******************************************************************************
 * Copyright (c) 2018 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
// import * as vscode from "vscode";
const MCLog_1 = require("./MCLog");
const Logger_1 = require("../../Logger");
const translator_1 = require("../../constants/strings/translator");
class AppLog extends MCLog_1.default {
    constructor(projectID, projectName, managerMap) {
        super(projectID, projectName, MCLog_1.default.LogTypes.APP, managerMap, translator_1.default.t(MCLog_1.default.STRING_NS, "waitingForAppLogs", { projectName }));
        this.projectID = projectID;
        this.projectName = projectName;
        this.initialized = false;
        this.previousLength = 0;
        // update will be invoked when we get a container-logs event
    }
    update(contents) {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.doUpdate) {
                Logger_1.default.e("Update was invoked on an applog with doUpdate=false, this should never happen!");
            }
            if (!this.initialized) {
                this.initialized = true;
                this.outputChannel.clear();
            }
            let newContents;
            const diff = contents.length - this.previousLength;
            if (diff === 0) {
                // no new output
                return;
            }
            else if (diff < 0) {
                // the log was cleared, eg due to container restart
                this.outputChannel.clear();
                newContents = contents;
            }
            else {
                // get only the new output
                newContents = contents.substring(this.previousLength, contents.length);
            }
            this.outputChannel.append(newContents);
            this.previousLength = contents.length;
            this.onChange();
        });
    }
}
exports.default = AppLog;
//# sourceMappingURL=AppLog.js.map