"use strict";
/*******************************************************************************
 * Copyright (c) 2018, 2019 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v2.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v20.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const TreeItemAdaptable_1 = require("./TreeItemAdaptable");
const ConnectionManager_1 = require("../microclimate/connection/ConnectionManager");
const Resources_1 = require("../constants/Resources");
const Commands_1 = require("../constants/Commands");
const translator_1 = require("../constants/strings/translator");
const StringNamespaces_1 = require("../constants/strings/StringNamespaces");
const STRING_NS = StringNamespaces_1.default.TREEVIEW;
class ProjectTreeDataProvider {
    constructor() {
        this.treeDataProvider = this;
        this.VIEW_ID = "ext.mc.mcProjectExplorer"; // must match package.json
        this.onChangeEmitter = new vscode.EventEmitter();
        this.onDidChangeTreeData = this.onChangeEmitter.event;
        /**
         * Notifies VSCode that this tree has to be refreshed.
         * Used as a call-back for ConnectionManager OnChange.
         */
        this.refresh = () => {
            // Logger.log("Refresh tree");
            this.onChangeEmitter.fire();
        };
        ConnectionManager_1.default.instance.addOnChangeListener(this.refresh);
    }
    /**
     * TreeDataProvider method to convert our custom TreeItemAdaptable class to a vscode.TreeItem
     */
    getTreeItem(node) {
        return node.toTreeItem();
    }
    /**
     * TreeDataProvider method to get children for a given TreeItemAdaptable node, or provide the tree's root node.
     */
    getChildren(node) {
        if (node == null) {
            const connections = ConnectionManager_1.default.instance.connections;
            if (connections.length > 0) {
                // The top-level nodes of this tree are our Connections, and their children are their Projects
                return connections;
            }
            else {
                // Provide a root node if no Connections have been created
                const noConnectionsRoot = new TreeItemAdaptable_1.SimpleTreeItem(translator_1.default.t(STRING_NS, "noConnectionsLabel"), vscode.TreeItemCollapsibleState.None);
                noConnectionsRoot.treeItem.iconPath = Resources_1.default.getIconPaths(Resources_1.default.Icons.Microclimate);
                noConnectionsRoot.treeItem.tooltip = translator_1.default.t(STRING_NS, "noConnectionsTooltip");
                // Clicking the no connections item runs the new connection command.
                noConnectionsRoot.treeItem.command = {
                    command: Commands_1.default.NEW_CONNECTION,
                    title: "" // non-nls
                };
                return [noConnectionsRoot];
            }
        }
        else {
            return node.getChildren();
        }
    }
}
exports.default = ProjectTreeDataProvider;
//# sourceMappingURL=ProjectTree.js.map